 # Train License Plate Recognition on a Custom Dataset
 Here we describe how to finetune Hailo's optical character reader (OCR) model for license plate recognition with your own custom dataset.
## Prerequisites
* docker ([installation instructions](https://docs.docker.com/engine/install/ubuntu/))
* nvidia-docker2 ([installation instructions](https://docs.nvidia.com/datacenter/cloud-native/container-toolkit/install-guide.html))
> **_NOTE:_**  In case you use Hailo Software Suite docker, make sure you are doing all the following instructions outside this docker.

## Environment Preparations

1. **Build the docker image**
    ```
    cd hailo_model_zoo/hailo_models/license_plate_recognition/
    docker build  --build-arg timezone=`cat /etc/timezone` -t license_plate_recognition:v0 .
    ```
    - This command will build the docker image with the necessary requirements using the Dockerfile that exists in this directory.

2. **Start your docker:**
    ```
    docker run --name <your_docker_name> -it --gpus all --ipc=host -v /path/to/local/drive:/path/to/docker/dir license_plate_recognition:v0
    ```
      - `docker run` create a new docker container.
      - `--name <your_docker_name>` name for your container.
      - `-it` runs the command interactively.
      - `--gpus all` allows access to all GPUs.
      - `--ipc=host` sets the IPC mode for the container.
      - `-v /path/to/local/data/dir:/path/to/docker/data/dir` maps `/path/to/local/data/dir` from the host to the container. You can use this command multiple times to mount multiple directories.
      - `license_plate_recognition:v0` the name of the docker image.

## Finetuning and exporting to ONNX
1. **Train the network on your dataset**<br>
Once the docker is started, you can train the OCR on your custom dataset.

    - Create a folder with license plates images for training and testing. The folder should contain images whose file names correspond to the plate number, e.g. `12345678.png`.
        > **_NOTE:_**  Please make sure the file names **do not** contain characters which are not numbers or letters.
    - Alternatively, you can use the provided jupyter notebook in `dataset/lp_autogenerate.ipynb` as an example of how to auto-generate a synthetic dataset of license plates for training. The auto-generation uses several parameters to control the randomness in the dataset, such as allowed characters, font size, font color, character separation etc. (Please refer to the notebook for more details). In order to auto-generate the synthetic dataset, please provide the following:
        - Clean license plate images with no characters in the  `dataset/plates/` folder
        - `.ttf` font files in the `dataset/fonts/` folder

        > **_NOTE:_**  We recommend the autogenerated training set to contain at least 4 million images

    - Start training on your dataset:
        - Start from our pre-trained weights in `pre_trained/lprnet.pth` (you can also download it from [here](https://hailo-model-zoo.s3.eu-west-2.amazonaws.com/HailoNets/LPR/ocr/lprnet/2022-03-09/lprnet.pth))
            ```
            python train_LPRNet.py --train_img_dirs path/to/train/images --test_img_dirs path/to/test/images --max_epoch 30 --resume_epoch 15 --pretrained_model pre_trained/lprnet.pth --save_folder runs/exp0/
            ```
        - Or train from scratch
            ```
            python train_LPRNet.py --train_img_dirs path/to/train/images --test_img_dirs path/to/test/images --max_epoch 15 --save_folder runs/exp0/
            ```

2. **Export to ONNX**<br>
Export the model to ONNX using the following command:
    ```
    python export.py --onnx lprnet.onnx --weights /path/to/trained/model.pth
    ```
<br>

---
## Compile the Model using Hailo Model Zoo<br>
You can generate an HEF file for inference on Hailo-8 from your trained ONNX model. In order to do so you need a working model-zoo environment.
Choose the model YAML from our networks configuration directory, i.e. `hailo_model_zoo/cfg/networks/lprnet.yaml`, and run compilation using the model zoo:
```
python hailo_model_zoo/main.py compile --ckpt lprnet.onnx --calib-path /path/to/calibration/imgs/dir/ --yaml lprnet.yaml
```

* <code>--ckpt</code> - path to your ONNX file.
* <code>--calib-path</code> - path to a directory with your calibration images in JPEG/PNG format
* <code>--yaml</code> - path to your configuration YAML file.
<br>

The model zoo will take care of adding the input normalization to be part of the model. The compiled model can be used in the [**Hailo TAPPAS**](https://hailo.ai/developer-zone/tappas-apps-toolkit/) to generate a full application.